#!/bin/bash

# # shellcheck disable=SC2154,SC1091,SC2155
# This helper script will handle generation of a secret in the app namespace
#
# /scripts/gen-user-secret.sh
#

set -e

source /scripts/utils.sh

check_access ${infraNamespace} secrets
check_access ${appsNamespace} secrets

if [ -z "${dbOverrideUserList}" ]; then
  userList="${namingPrefix}-user"
else
  userList="${dbOverrideUserList}"
fi

# Set timeout for psql cmd looping: 120s for postgres. 0 secs for aws mode ( no checking loop. execute once only)
if [ ${awsDeployment} == "true" ]; then
  ckTimeout=0
else
  ckTimeout=120
fi

for username in ${userList}; do
  if [ ! -z "${dbOverrideUserList}" ]; then
    dbUserCredsSecretPassName="${namingPrefix}-${username}-pg-pass"
    dbUserCredsSecretUserName="${namingPrefix}-${username}-pg-user"
    username="${namingPrefix}-${username}-user"
  fi
  cprint "Checking for secret.."
  if /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" get secret "${dbUserCredsSecretPassName}"; then
    cprint "Secret ${dbUserCredsSecretPassName} already exists."
  else
    cprint "No secret found for ${dbUserCredsSecretPassName}.. Generating new account."
    cprint "Adding user ${username} to ${dbServiceClusterEndpoint}"
    password=$(gen_password)
    # Change SQL Query for AWS vs Localstack
    if [ ${awsDeployment} == "true" ]; then
      SQL=$(cat <<-END
SET password_encryption = 'scram-sha-256';
CREATE USER "${username}" WITH PASSWORD '${password}';
ALTER USER "${username}" WITH PASSWORD '${password}';
GRANT ALL PRIVILEGES ON DATABASE "${databaseName}" TO "${username}";
GRANT rds_replication TO "${username}";
CREATE SCHEMA IF NOT EXISTS "${defaultSchema}";
GRANT ALL PRIVILEGES ON SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO cbadmin;
CREATE ROLE rds_pgaudit;
CREATE EXTENSION pgaudit;
ALTER DATABASE "${databaseName}" SET pgaudit.log='All';
END
)
    else
      SQL=$(cat <<-END
SET password_encryption = 'scram-sha-256';
CREATE USER "${username}" WITH PASSWORD '${password}';
GRANT ALL PRIVILEGES ON DATABASE "${databaseName}" TO "${username}";
CREATE SCHEMA IF NOT EXISTS "${defaultSchema}";
GRANT ALL PRIVILEGES ON SCHEMA "${defaultSchema}" TO "${username}";
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA "${defaultSchema}" TO "${username}";
END
)
    fi
    cat >/tmp/data.sql <<EOL
${SQL}
EOL
    /scripts/wait-until -t "${ckTimeout}" -i 5 true psql -h "${dbServiceClusterEndpoint}" -p ${port} -U cbadmin -d "${databaseName}" -f /tmp/data.sql
    echo "Creating kubernetes secret ${dbUserCredsSecretUserName}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" delete secret "${dbUserCredsSecretUserName}" --ignore-not-found
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" create secret generic "${dbUserCredsSecretUserName}" --from-literal=user="${username}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" create secret generic "${dbUserCredsSecretUserName}" --from-literal=user="${username}"
    echo "Creating kubernetes secret ${dbUserCredsSecretPassName}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${appsNamespace}" create secret generic "${dbUserCredsSecretPassName}" --from-literal=password="${password}"
    /scripts/wait-until -t 30 -i 5 true kubectl --namespace "${infraNamespace}" create secret generic "${dbUserCredsSecretPassName}" --from-literal=password="${password}"
  fi
done

print_header "$0 done."
